unit ssd1305u;

interface

uses
  Math,
  ssd1305hu,
  wiringpi;

procedure SSD1305_begin();

procedure SSD1305_clear();

procedure SSD1305_pixel(X     : Integer;
                        Y     : Integer;
                        Color : Byte);

procedure SSD1305_char1616(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);

procedure SSD1305_char3216(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);

procedure SSD1305_char(X     : Byte;
                       Y     : Byte;
                       Ascii : Char;
                       Size  : Byte;
                       Mode  : Byte);

procedure SSD1305_string(X    : Byte;
                         Y    : Byte;
                         Str  : PChar;
			 Size : Byte;
                         Mode : Byte);

procedure SSD1305_bitmap(X        : Byte;
                         Y        : Byte;
                         PBmp     : PByte;
                         ChWidth  : Byte;
                         ChHeight : Byte);

procedure SSD1305_display();

implementation

const
 CHANNEL  = 0;
 WIDTH    = 128;
 HEIGHT   = 32;
 RST      = 6;
 DC       = 5;

var
 buffer : array [0 .. 511] of Byte;

procedure command(Cmd : Byte);
begin
 digitalWrite(DC,LOW);
 wiringPiSPIDataRW(CHANNEL, @Cmd, 1);
end;

procedure SSD1305_begin();
begin
 wiringPiSetup();
 pinMode(RST,OUTPUT);
 pinMode(DC,OUTPUT);
 wiringPiSPISetup(CHANNEL, 2000000);    //2M

 digitalWrite(RST,HIGH);
 delay(10);
 digitalWrite(RST,LOW);
 delay(10);
 digitalWrite(RST,HIGH);

 command($AE);  //--turn off oled panel
 command($04);  //--turn off oled panel
 command($10);  //--turn off oled panel
 command($40);  //---set low column address
 command($81);  //---set high column address
 command($80);  //--set start line address  Set Mapping RAM Display Start Line ($00~$3F)
 command($A1);  //--set contrast control register
 command($A6);  // Set SEG Output Current Brightness
 command($A8);  //--Set SEG/Column Mapping     $a0Г—ГіГ“Г’В·ВґГ–Гѓ $a1Г•ГЅВіВЈ
 command($1F);  //Set COM/Row Scan Direction   $c0Г‰ГЏГЏГ‚В·ВґГ–Гѓ $c8Г•ГЅВіВЈ
 command($C8);  //--set normal display
 command($D3);  //--set multiplex ratio(1 to 64)
 command($00);  //--1/64 duty
 command($D5);  //-set display offset	Shift Mapping RAM Counter ($00~$3F)
 command($F0);  //-not offset
 command($d8);  //--set display clock divide ratio/oscillator frequency
 command($05);  //--set divide ratio, Set Clock as 100 Frames/Sec
 command($D9);  //--set pre-charge period
 command($C2);  //Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
 command($DA);  //--set com pins hardware configuration
 command($12);
 command($DB);  //--set vcomh
 command($08);  //Set VCOM Deselect Level
 command($AF);  //-Set Page Addressing Mode ($00/$01/$02)
end;

procedure SSD1305_clear();
begin
 Initialize(buffer);
end;

procedure SSD1305_pixel(X     : Integer;
                        Y     : Integer;
                        Color : Byte);
 var
  bufIndex : Integer;

begin
 if not ((X > WIDTH) or (Y > HEIGHT)) then
 begin
  bufIndex := x + (y div 8) * WIDTH;

  if ((bufIndex >= 0) and (bufIndex < 512)) then
  begin
   if (color = 1) then
   begin
    buffer[bufIndex] := buffer[bufIndex] or (1 shl (y mod 8));
   end
    else
   begin
    buffer[bufIndex] := buffer[bufIndex] and (255 - (1 shl (y mod 8)));
   end;
  end;
 end;
end;

procedure SSD1305_char1616(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);
 var
  i,j    : Byte;
  chTemp : Byte;
  y0     : Byte;
  chMode : Byte;
  b      : Byte;

begin
 chTemp := 0;
 y0     := y;
 chMode := 0;

 for i := 0 to 31 do
 begin
  chTemp := Font1612[chChar - $30][i];
  b      := $80;

  for j := 0 to 7 do
  begin
   chMode := IfThen(Boolean(chTemp and b),1,0);

   SSD1305_pixel(x, y, chMode);

   b := b div 2;

   Inc(y);

   if ((y - y0) = 16) then
   begin
    y := y0;
    Inc(x);

    Break;
   end;
  end;
 end;
end;

procedure SSD1305_char3216(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);
 var
  i,j    : Byte;
  chTemp : Byte;
  y0     : Byte;
  chMode : Byte;
  b      : Byte;

begin
 chTemp := 0;
 y0     := y;
 chMode := 0;

 for i := 0 to 63 do
 begin
  chTemp := Font3216[chChar - $30][i];
  b      := $80;

  for j := 0 to 7 do
  begin
   chMode := IfThen(Boolean(chTemp and b),1,0);

   SSD1305_pixel(x, y, chMode);

   b := b div 2;

   Inc(y);

   if ((y - y0) = 32) then
   begin
    y := y0;
    Inc(x);

    Break;
   end;
  end;
 end;
end;

procedure SSD1305_char(X     : Byte;
                       Y     : Byte;
                       Ascii : Char;
                       size  : Byte;
                       Mode  : Byte);
 var
  i,j,y0 : Byte;
  temp   : Byte;
  ch     : Byte;
  b      : Byte;

begin
 y0 := y;
 ch := Ord(Ascii) - Ord(' ');

 for i := 0 to Size - 1 do
 begin
  if (Size = 12) then
  begin
   if (Mode = 1) then temp := Font1206[ch][i]
                 else temp := 255 - Font1206[ch][i];
  end
   else
  begin
    if (Mode = 1) then temp := Font1608[ch][i]
                  else temp := 255 - Font1608[ch][i];
  end;

  b := $80;

  for j := 0 to 7 do
  begin
   if Boolean(temp and b) then SSD1305_pixel(x,y,1)
                          else SSD1305_pixel(x,y,0);
   b := b div 2;

   Inc(y);

   if ((y - y0) = Size) then
   begin
    y := y0;
    Inc(x);

    break;
   end;
  end;
 end;
end;

procedure SSD1305_string(X    : Byte;
                         Y    : Byte;
                         Str  : PChar;
			 Size : Byte;
                         Mode : Byte);
 var
  pChr : PChar;
  n    : Integer;

begin
 pChr := PChar(Str);
 n    := 0;

 while (n <> Length(Str)) do
 begin
  if (x > (WIDTH - Size div 2)) then
  begin
   x := 0;
   y := y + Size;

   if (y > (HEIGHT - Size)) then
   begin
    y := 0;
    x := 0;
   end;
  end;

  SSD1305_char(x, y, pChr^, Size, Mode);

  x := x + Size div 2;

  Inc(pChr);
  Inc(n)
 end;
end;

procedure SSD1305_bitmap(X        : Byte;
                         Y        : Byte;
                         PBmp     : PByte;
                         ChWidth  : Byte;
                         ChHeight : Byte);
 var
  i,j       : Byte;
  byteWidth : Byte;

begin
 byteWidth := (ChWidth + 7) div 8;

 for j := 0 to ChHeight - 1 do
 begin
  for i := 0 to ChWidth - 1 do
  begin
   if Boolean(PByte(PBmp + j * byteWidth + i div 8)^ and (128 shr (i and 7))) then
   begin
    SSD1305_pixel(x + i, y + j, 1);
   end;
  end;
 end;
end;

procedure SSD1305_display();
 var
  page : Byte;
  pBuf : PByte;

begin
 pBuf := @buffer[0];

 for page := 0 to 3 do
 begin
  command($B0 + page);     // set page address
  command($04);            // set low column address
  command($10);            // set high column address

  // write data

  digitalWrite(DC, HIGH);
  wiringPiSPIDataRW(CHANNEL, pBuf, WIDTH);

  Inc(pBuf,WIDTH);
 end;
end;

end.

